/**
 * Header       LcdMenu.h
 * Author       2021-02-02 Charles Geiser (https://www.dodeka.ch)
 * 
 * Purpose      Declaration of the class LcdMenu with its methods
 *                  nextMenuItem()
 *                  prevMenuItem()
 *                  show()
 *                  doAction() 
 *               
 * Constructor
 * arguments    &lcd                   A reference to an instance of the class LiquidCrystal_I2C
 *              menu[]                 Array of menu items consisting of pairs of { itemText, itemAction }
 *              nbrMenuItems           Number of menu items contained in menu
 *              nbrDisplayedMenuItems  Number of displayed menu items on LCD (possibly less than the number of LCD rows)
 */

#include <LiquidCrystal_I2C.h>

#ifndef _LCDMENU_H_
#define _LCDMENU_H_

// Some special characters and their octal codes
// (HD44780 controller with english/japanese font)
#define LCD_aUml    "\341"
#define LCD_oUml    "\357"
#define LCD_uUml    "\365"
#define LCD_ARROW_R "\176"
#define LCD_ARROW_L "\177"
#define LCD_DOT     "\245"
#define LCD_DEG     "\337"
#define LCD_alpha   "\340"
#define LCD_beta    "\342"
#define LCD_epsilon "\343"
#define LCD_mu      "\344"
#define LCD_sigma   "\345"
#define LCD_rho     "\346"
#define LCD_SQRT    "\348"
#define LCD_INF     "\363"
#define LCD_OMEGA   "\364"
#define LCD_SIGMA   "\366"
#define LCD_pi      "\367"
#define LCD_DIV     "\375"
#define LCD_CURSOR  "\377"

// Some special characters and their octal codes
// (HD44780 controller with european font)
#define LCDE_aUml    "\204"
#define LCDE_oUml    "\224"
#define LCDE_uUml    "\201"
#define LCDE_ARROW_U "\305" 
#define LCDE_ARROW_D "\306"
#define LCDE_ARROW_R "\307"
#define LCDE_ARROW_L "\310"
#define LCDE_DOT     "\315"
#define LCDE_DEG     "\262"
#define LCDE_alpha   "\337"
#define LCDE_beta    "\344"
#define LCDE_gamma   "\341"
#define LCDE_GAMMA   "\324"
#define LCDE_delta   "\342" 
#define LCDE_epsilon "\343"
#define LCDE_eta     "\345" 
#define LCDE_kappa   "\350"
#define LCDE_lambda  "\351"
#define LCDE_mu      "\352"
#define LCDE_nu      "\353"
#define LCDE_rho     "\356"
#define LCDE_SQRT    "\276"
#define LCDE_INF     "\302"
#define LCDE_omega   "\364"
#define LCDE_OMEGA   "\366"
#define LCDE_sigma   "\357"
#define LCDE_SIGMA   "\332"
#define LCDE_tau     "\360"
#define LCDE_pi      "\355"
#define LCDE_PHI     "\334"
#define LCDE_PSI     "\335"
#define LCDE_DIV     "\270"
#define LCDE_CURSOR  "\374"

typedef struct { const char *txt; void (&action)(); } MenuItem;
extern MenuItem menu[];
extern const uint8_t nbrMenuItems;

class LcdMenu
{
  public:
    LcdMenu(LiquidCrystal_I2C &lcd, MenuItem menu[], int nbrMenuItems, int nbrDisplayedMenuItems = 2) : 
      _lcd(lcd), 
      _menu(menu), 
      _nbrMenuItems(nbrMenuItems),
      _nbrDisplayedMenuItems(nbrDisplayedMenuItems)
    { 
      _menuPage = _selectedMenuItem / _nbrDisplayedMenuItems;
      _startMenuItem = _menuPage * _nbrDisplayedMenuItems;
      _stopMenuItem = _startMenuItem + _nbrDisplayedMenuItems;
    }

    void nextMenuItem();    // Selects the next menu item
    void prevMenuItem();    // Select the previous menu item
    void show(bool clearScreen = true);  // show a menu page on display
    void doAction();        // Calls the action associated wis the selected menu item

  private:
    LiquidCrystal_I2C &_lcd;
    MenuItem *_menu;   // Pointer to the array of menuitems
    uint8_t  _nbrMenuItems;
    int      _selectedMenuItem = 0;
    int      _startMenuItem = 0;
    int      _stopMenuItem;
    int      _menuPage = 0;
    int      _previousMenuPage = 0;
    int      _nbrDisplayedMenuItems;
};
#endif