/**
 * Program      lcdeMenu.cpp
 * Author       2021-04-18 Charles Geiser (www.dodeka.ch)
 * 
 * Purpose      The example shows how to define a menu array consisting of multiple
 *              pairs of { "itemText", itemAction } and pass it to the class LcdMenu.
 *              
 *              The LcdMenu is passed to the LcdMenuControlButton class, which then
 *              enables navigation through the menu by implementing the following methods:
 *                  onClick()          selects next menu item or returns to menu after action is done
 *                  onDoubleClick()    selects the previous menu item
 *                  onLongClick()      executes the action assigned to the selected menu item.
 * 
 * Board        Arduino UNO / Wemos D1 R2 ESP8266 / ESP32
 *
 * 
 * Wiring                12V            USB 
 *                   .--(   )-----------I I-----.
 *                   |                  ```  [o]|                 .----------------------.
 *                   |                          |             GND-+ .------------------. |
 *                   |    WEMOS D1 R2           |             Vcc-+ |    LCD 16 x 2    | | 
 *                   |                   SCL/D1 o             SDA-+ |       I2C        | |
 *                   |                   SDA/D2 o             SCL-+ '------------------' |
 *                   |                       NC o                 '----------------------'
 *                   |                      GND o
 *                   o IOREF             SCK/D5 o
 *                   o IOREF            MISO/D6 o
 *                   o RST              MOSI/D7 o
 *                   o 3.3V               SS/D8 o
 *              Vcc  o 5V                    D7 o     _I_  BTN_STOP
 *                   o GND                   D6 o-----o o---- GND
 *                   o GND  ...........         |     _I_
 *                   o Vin I           I     D5 o-----o o---- GND
 *                   |     I  ESP8266  I     D4 o          BTN_MENU
 *                   o A0  I           I     D3 o
 *                   o NC  I           I SDA/D2 o
 *                   o NC  I           I SCL/D1 o
 *                   o NC  I...........I     D0 o
 *                   o SDA |  _   _  | |     TX o
 *                   o SCL |_| |_| |_|_|     RX o
 *                   `--------------------------´ 
 * 
 *  
 * Remarks      All menu actions must have the same signature, namely: void action(void).
 * 
 * Warning      The default environment in platformio.ini is set to d1_mini (Wemos D1 R2 and D1 Mini)
 *              and we have to use the native GPIO numbers instead of pin numbers:
 *              D0 --> 16
 *              D1 --> 5
 *              D2 --> 4
 *              D3 --> 0
 *              D4 --> 2
 *              D5 --> 14
 *              D6 --> 12
 *              D7 --> 13
 *              D8 --> 15
 *              For Arduino UNO change the constants PIN_BTN_MENU and PIN_BTN_STOP  
 * References
 */
#include <Arduino.h>
#include "LcdMenu.h"
#include "LcdMenuControlButton.h"

#define COMPORT_SPEED  115200

#define LCD_ROWS         2    // or 4 for a LCD with 4 rows
#define LCD_COLUMNS      16   // or 20 for a LCD with 20 columns
#define LCD_ADDRESS      0x27 // I2C address

const uint8_t PIN_BTN_MENU = 14;   // for menu navigation and action triggering (Wemos D1/R2  14 = D5)
const uint8_t PIN_BTN_STOP = 12;   // for pausing action showAllChars           (Wemos D1/R2  12 = D6)

const char PRE_TXT1[] = "LCD Menu Demo";
const char PRE_TXT2[] = "by Carlo47";

// Forward declaration of menu actions
void action1();
void action2();
void action3();
void action4();
void action5();
void showAllChars();
void pong();
void showPreamble();

// Define the menu consisting of text and associated action
MenuItem menu[] =           
{
  { "Addition       ", action1 },
  { "Multiplication ", action2 },
  { "Division       ", action3 },
  { "Exponentiation ", action4 },
  { "Pythagoras     ", action5 },
  { "ShowAllChars   ", showAllChars },
  { "Pong           ", pong },
  { "Preamble       ", showPreamble }
};

// Calculate the number of menuitems
const uint8_t nbrMenuItems = sizeof(menu) / sizeof(menu[0]);

LiquidCrystal_I2C    lcd(LCD_ADDRESS, LCD_COLUMNS, LCD_ROWS);     // the LCD
LcdMenu              lcdMenu(lcd, menu, nbrMenuItems, LCD_ROWS);  // the menu
LcdMenuControlButton menuButton(PIN_BTN_MENU, lcdMenu);           // the menu button 

// Called by action1
uint16_t addition(uint16_t a, uint16_t b)
{
  return a + b;   
}

// Called by action 2
uint16_t multiplication(uint16_t a, uint16_t b)
{ 
  return a * b;
}

// Called by action 3
float division(float a, float b)
{
  return a / b;
}

// Called by action 4
float power(float a, float b)
{
  return pow(2, 14);
}

// Called by action 5
// Calculates the hypotenuse of a rectangular triangle
float pythagoras(float a, float b)
{
  return sqrt(a*a + b*b);
}

void action1()
{
  lcd.clear();
  lcd.print("13 + 14 = ");lcd.print(addition(13, 14));
}

void action2()
{
  lcd.clear();
  lcd.print("13 * 14 = ");lcd.print(multiplication(13, 14));
}

void action3()
{
  lcd.clear();
  lcd.print("355 / 113 = ");lcd.print(division(355, 113));
}

void action4()
{
  lcd.clear();
  lcd.print("2^14 = ");lcd.print(power(2, 14), 0);
}

void action5()
{
  lcd.clear();
  lcd.print("3, 4 = ");lcd.print(pythagoras(3, 4));
}

/**
 * action6
 * Show the character set of the LCD in multiple screens
 * 4 characters in the 1st and their octal code in the 2nd row
 * 1st:   |   }   ->  <-
 * 2nd:   174 175 176 177
 */
void showAllChars()
{
  lcd.clear();
  for (int i = 0; i < 256; i++)
  {
    int c = (i % 4);
    lcd.setCursor(c * 4, 0);
    lcd.write((char)i);        // print char in 1st row
    lcd.setCursor(c * 4, 1);
    lcd.print(i, 8);           // print its octal code in 2nd row
    if (c == 3)
    {
      lcd.display();                           // display line when 4 chars are printed
      delay(1000);                             // and wait 0.5 sec or
      while (digitalRead(PIN_BTN_STOP) == LOW)  // as long as the button is pressed
        { delay(10); }                           
    }
  }
}

/**
 * action7
 * Scroll an arrow in first row from left to right
 * and in second row back from right to left
 */
void pong()
{
  lcd.clear();
  lcd.print(LCD_ARROW_R);
  for (int i = 0; i < LCD_COLUMNS-1; i++)
  {
    lcd.scrollDisplayRight();
    delay(150);
  }
  lcd.clear();
  lcd.setCursor(LCD_COLUMNS-1, 1);
  lcd.print(LCD_ARROW_L);
  delay(150);
  for (int i = 0; i < LCD_COLUMNS-1; i++)
  {
    lcd.scrollDisplayLeft();
    delay(150);
  }
  lcd.clear();
  lcd.print("Catched!");
}

/**
 * Show two rows of text as preamble for msWait milliseconds
 */ 
void preamble(const char *txt1, const char *txt2, uint32_t msWait=2000)
{
  lcd.clear();
  lcd.print(txt1);  
  lcd.setCursor(0,1);
  lcd.print(txt2); 
  lcd.display();
  delay(msWait);
  for (int i = 0; i < LCD_COLUMNS; i++) 
  {
    lcd.scrollDisplayRight();
    delay(150);
  }
  delay(1000);
}

/**
 * action8
 * Show preamble and scroll it right over the screen.
 * On a 4x20 LCD it reapears from left on rows 2 and 3
 */
void showPreamble()
{
  preamble(PRE_TXT1, PRE_TXT2);
}

void initLcd()
{
  lcd.init();
  lcd.backlight();
  preamble(PRE_TXT1, PRE_TXT2);
  Serial.println("initLcd(): ===> done");
}

void setup() 
{
  Serial.begin(COMPORT_SPEED);
  pinMode(PIN_BTN_STOP, INPUT_PULLUP);
  initLcd();
  lcdMenu.show(true);
  Serial.println("setup(): ===> done");
}

void loop() 
{
  menuButton.loop(); // handle button clicks
}
